/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include "CameraSetting.h"

#include "controllers/common/family/Families.h"
#include <boost/lexical_cast.hpp>

using namespace boost;
using namespace std;

namespace camera {

const std::string CameraSetting::TYPE = "Camerasetting";
 const std::string CameraSetting:: BINNING[5] = {"NONE", "1*1","2*2","NONE","4*4"};
 const std::string CameraSetting:: MODE[5] = {"ND", "Single","ND","Multi","ND"};

CameraSetting::CameraSetting(const string& name) :
	ExperimentController(name), controller::Start(this) {

	setFamily(family::ACQUISITION, family::SETTING);

	hbinning.init(this, SAVE , "actualhBinning", "wantedhBinning");
	vbinning.init(this, SAVE , "actualvBinning", "wantedvBinning");
//	hflip.init(this, SAVE, "hflip");
//	vflip.init(this, SAVE, "vflip");
//	rotate.init(this, SAVE, "rotate");
	mode.init(this, SAVE, "actualmode", "wantedmode");
	nbKine.init(this, SAVE, "nbKine");
	kineTime.init(this, SAVE, "kineTime");
	modeFVB.init(this, SAVE, "modeFVB");

	rotate.init(this, SAVE, "rotate");
	cooling.init(this, SAVE, "cooling");
	readout.init(this, SAVE, "readout");
	shutopentime.init(this, SAVE, "shutopentime");
	shutclosetime.init(this, SAVE, "shutclosetime");
	shutmode.init(this, SAVE, "shutmode");
	hsspeed.init(this, SAVE, "hsspeed");
	gain.init(this, SAVE, "gain");
	bclamp.init(this, SAVE, "bclamp");
	tempstatus.init(this, SAVE, "tempstatus");
	vsspeed.init(this, SAVE, "vsspeed");


	stringvbinning.init(this, NOSAVE| SPY, "actualstringvbin", "wantedstringvbin", "vBinn");
	stringhbinning.init(this, NOSAVE| SPY, "actualstringhbin", "wantedstringhbin", "hBinn");
	stringmode.init(this, NOSAVE| SPY, "actualstringmode", "wantedstringmode",  "Frame");
	temperature.init(this, SAVE| SPY, "actualtemp", "wantedtemp", "Temp");
	temperature.setUnit("~degree");

	m_CamDriver.init(this, "cammeraDriver");
	detectorController.init(this, "detectorController");
	detectorElement.init(this, "detectorElement");



	registerFunction(TYPE);




}

CameraSetting::~CameraSetting() {
}

void CameraSetting::postConfiguration() {
	hbinning=m_CamDriver->hbind();
	//stringhbinning=BINNING[hbinning()];
	vbinning=m_CamDriver->vbind();
	//stringvbinning=BINNING[vbinning()];

	mode=m_CamDriver->acqmode();
	temperature=m_CamDriver->actualTemperature();
	//stringmode=MODE[mode()];

	registerRefresher(hbinning, &CameraSetting::refreshBinning, this);
	registerRefresher(vbinning, &CameraSetting::refreshBinning, this);
	registerRefresher(mode, &CameraSetting::refreshMode, this);
	registerUpdater(m_CamDriver->hbind, &CameraSetting::updateBinning, this);
	registerUpdater(m_CamDriver->vbind, &CameraSetting::updateBinning, this);
	registerUpdater(m_CamDriver->acqmode, &CameraSetting::updateMode, this);
	registerUpdater(m_CamDriver->actualTemperature, &CameraSetting::updateTemperature, this);

}

void CameraSetting::start() {
	// Test speed BOB
	//hbinning.setpoint = 1;
	//vbinning.setpoint = 2088;
	if (modeFVB() == true) {
		hbinning.setpoint = 1;
		vbinning.setpoint = 2088;
	}else {
		hbinning.setpoint = 1;
		vbinning.setpoint = 1;
	}



	int hvalue = hbinning.setpoint();
	m_CamDriver->hbind = hvalue;
	int vvalue = vbinning.setpoint();
	m_CamDriver->vbind = vvalue;

	m_CamDriver->acqmode = mode.setpoint();
//	m_CamDriver->hflip = hflip();
//	m_CamDriver->vflip = vflip();
	m_CamDriver->nbSlicesAcq = nbKine();
	m_CamDriver->modeFVB = modeFVB();
	//m_CamDriver->time = kineTime();
	if ((vvalue != 0) || (hvalue != 0)) {

		if (modeFVB() == false) {
			cout <<  "modeFVB == false hvalue " << hvalue << endl;
			cout <<  "modeFVB == false vvalue " << vvalue << endl;

			detectorController->detSize = (MAX_XSIZE / hvalue * MAX_YSIZE
					/ vvalue);
			//detectorController->detSize=(MAX_XSIZE *MAX_YSIZE);
			detectorElement->xSize = (MAX_XSIZE / hvalue);
			detectorElement->ySize = (MAX_YSIZE / vvalue);
		} else {
			cout <<  "modeFVB == true " << endl;
			detectorController->detSize = (2088);
			detectorElement->xSize = (2088);
			detectorElement->ySize = (1);
			detectorElement->wSize = (1);
		}

	}

	if (mode() == 3) {
		if ((vvalue != 0) || (hvalue != 0)) {

			detectorElement->wSize = nbKine();
			detectorController->detSize = (MAX_XSIZE / hvalue * MAX_YSIZE
					/ vvalue) * nbKine();
			detectorController->wSize = nbKine();
		}
	}

	m_CamDriver->execute("writeParam");

	rotate = m_CamDriver->rotate();
	cooling= m_CamDriver->cooling ();
	readout = m_CamDriver->readout ();
	shutopentime = m_CamDriver->shutopentime ();
	shutclosetime = m_CamDriver->shutclosetime ();
	shutmode = m_CamDriver->shutmode ();
	hsspeed = m_CamDriver->hsspeed ();
	bclamp = m_CamDriver->bclamp ();
	gain = m_CamDriver->gain ();
	tempstatus = m_CamDriver->tempstatus ();
	vsspeed = m_CamDriver->vsspeed ();

}


void CameraSetting::refreshBinning(int32 value) {


}

void CameraSetting::refreshMode(int32 value) {


}

void CameraSetting::updateBinning() {
//	hbinning=m_CamDriver->hbind();
//	stringhbinning=BINNING[hbinning()];
//	vbinning=m_CamDriver->vbind();
//	stringvbinning=BINNING[vbinning()];

}

void CameraSetting::updateTemperature() {
	temperature=m_CamDriver->actualTemperature();

}

void CameraSetting::updateMode() {

	mode=m_CamDriver->acqmode();
	stringmode=MODE[mode()];

}

}

