/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief Cyclops Par command
 * \author J.Locatelli
 * \date 16-04-2019
 */

#ifndef  CYCLOPSPARCMD_H
#define  CYCLOPSPARCMD_H

#include "controllers/dif/common/commandline/DiffParCmd.h"

namespace cyclops {

/*!
 * \class CyclopsParCmd
 * \brief Cyclops Par   command
 * \par
 * This class implement the tas set command.
 */
class CyclopsParCmd: public dif::DiffParCmd {

public:

	/*!
	 * \brief Constructor
	 */
	CyclopsParCmd(AbstractParser* aParser);

	/*!
	 * \brief Destructor
	 */
	virtual ~CyclopsParCmd();

	/*!
	 * \brief Decode the command line
	 *
	 * \param[in] items list of item contains on the command line
	 * \throws bad_any_cast bad cast on boost any property
	 * \throws BadInput syntax error when paring line
	 * \throws PropertyNotFound property not found
	 * \throws ControllerNotFound controller not found
	 * \throws CannotSetValue could not set property value
	 */
	virtual void parse(Items& items) throw (CannotSetValue, CommonCmdDef::BadInput);
};

}
#endif // CYCLOPSPARCMD_H
