/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef D22SPECTROSAMPLESEQUENCER_H_
#define D22SPECTROSAMPLESEQUENCER_H_

#include <Controller.h>
#include "controllers/lss/common/SampleChangerController.h"
#include "controllers/common/utilities/DataParams.h"
#include "CountWithSpectro.h"

namespace d22special {

class D22SpectroSampleSequencer: public ExperimentController, public controller::Stoppable {

public:

	static const std::string TYPE;
	static const int32 MAX_POSITIONS;

	static const char DATAPARAM_TRANSMISSION_PROPERTY[];
	static const char DATAPARAM_THICKNESS_PROPERTY[];

	D22SpectroSampleSequencer(const std::string & name);
	D22SpectroSampleSequencer(const D22SpectroSampleSequencer& controller);
	virtual ~D22SpectroSampleSequencer();
	void postConfiguration();

	void start(); // virtual?
	void stop();

	virtual void setEnabledForLoading(bool all, bool postconf = true);
	virtual void setDisabled(bool all = false);

private:

	void refreshChangerSetpoint(int32 number);
	void refreshAcquisitionTime(int32 index, float64 value) throw (CannotSetValue);
	void refreshNumSlotSetpoint(int32 number);

	ControllerPtr<utilities::DataParams> m_dataParams;
	ControllerPtr<lss::SampleChangerController> m_sampleChanger;
	ControllerPtr<CountWithSpectro> countWithSpectro;

	Property<int32> maxPos;
	Property<int32> maxSlot;
	Property<int32, SETPOINT> numSlot;

	Property<int32> changerNum;
	Property<int32, SETPOINT> changer;
	DynamicProperty<int32> changerValues;
	DynamicProperty<std::string> changerLabels;

	DynamicProperty<int32> samplePosition;
	DynamicProperty<float64> acquisitionTime;
	DynamicProperty<float64> transmission;
	DynamicProperty<float64> thickness;
	DynamicProperty<std::string> acquisitionTitle;

	Property<std::string> dataparamsThickness;
	Property<std::string> dataparamsTransmission;

	//specific countWithSpectro properties
	Property<std::string> timeType;
	Property<int32> integrationTime;
	Property<int32> numSpectroMeasures;
	Property<int32> scansToAverage;

	//Info properties
	Property<float64> totalTime;

	bool m_ChangerStarted;
	bool m_CountStarted;

	ArrayProperty<int32>  yDataArray1;
	ArrayProperty<int32>  yDataArraySecondChannel1;

	ArrayProperty<int32>  yDataArray2;
	ArrayProperty<int32>  yDataArraySecondChannel2;

	int32* m_yDataArray1;
	int32* m_yDataArraySecondChannel1;

	int32* m_yDataArray2;
	int32* m_yDataArraySecondChannel2;


};

}

#endif /* D22SPECTROSAMPLESEQUENCER_H_ */
