/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "controllers/common/family/Families.h"
#include "contrib/rapidjson/document.h"
#include "contrib/rapidjson/reader.h"
#include "contrib/rapidjson/writer.h"
#include "contrib/rapidjson/prettywriter.h"
#include "Nomad3DController.h"
#include "InstrumentManager/InstrumentManager.h"

using namespace boost;
using namespace std;
using namespace rapidjson;

namespace nomad3d {

const string Nomad3DController::TYPE = "nomad3d_controller";

Nomad3DController::Nomad3DController(const string& name) :
	ExperimentController(name), controller::Init(this) {

	setFamily(family::HIDDEN);

	axisPositions.init(this, NOSAVE, "axis_positions");
}

Nomad3DController::Nomad3DController(const Nomad3DController& controller) :
	ExperimentController(controller) {
}

Nomad3DController::~Nomad3DController() {
}

void Nomad3DController::postConfiguration() {
}

void Nomad3DController::init() {

	map<string, ExperimentController*> experimentControllersMap = InstrumentManager::getInstance()->getAllInstalledExperimentControllersMap();
	map<string, ExperimentController*>::iterator it;
	int i = 0;

	// Iterate the controllers.
	for (it = experimentControllersMap.begin(); it != experimentControllersMap.end(); it++) {

		// Get the AxisController controllers.
		if (dynamic_cast<axis::AxisController*>(it->second) != 0) {

			axis::AxisController* controller = dynamic_cast<axis::AxisController*>(it->second);
			controller->attach(this); //Attach it to in order to be able to receive propertyChanged events.
			m_axisControllers.push_back(controller);

			registerUpdater(controller->position, &Nomad3DController::updateActualPosition, this, i);
			++i;
		}

		// Get the VirtualAxisController controllers.
		if (dynamic_cast<axis::VirtualAxisController*>(it->second) != 0) {

			axis::VirtualAxisController* controller = dynamic_cast<axis::VirtualAxisController*>(it->second);
			controller->attach(this); //Attach it to in order to be able to receive propertyChanged events.
			m_virtualAxisControllers.push_back(controller);

			registerUpdater(controller->position, &Nomad3DController::updateActualPosition, this, i);
			++i;
		}
	}

	updateActualPosition(0);
}

void Nomad3DController::updateActualPosition(int32 i) {

	PrettyWriter<StringBuffer> writer(s);
	s.Clear();

	writer.StartObject();

	// Write the axis controllers.
	for (auto controller : m_axisControllers) {
		writer.Key(controller->getName().c_str());
		writer.Double(controller->position() - controller->position.offset());
	}

	// Write the virtual axis controllers.
	for (auto controller : m_virtualAxisControllers) {
		writer.Key(controller->getName().c_str());
		writer.Double(controller->position() - controller->position.offset());
	}

	writer.EndObject();

	axisPositions = s.GetString();
}

}
