/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENPksController.h"
#include "drivers/global/DriversCommands.h"
#include "controllers/common/family/Families.h"
#include <boost/thread/thread.hpp>

using namespace std;
using namespace boost;

namespace npp {

const string CAENPksController::TYPE = "pks_controller";

/*
 * Constructor
 */
CAENPksController::CAENPksController(const string& name) :
		ExperimentController(name) {

	setFamily(family::ACQUISITION, family::SETTING);

	nbChannels.init(this, NOSAVE, "nb_channels");
	channelsActive.init(this, NOSAVE, "channels_active");
	triggerThreshold.init(this, NOSAVE, "trigger_threshold");

	m_adcDriver.init(this, "pks_driver");

	registerFunction(TYPE);
}

/*
 * Destructor
 */
CAENPksController::~CAENPksController() {

}

/*
 * postConfiguration
 */
void CAENPksController::postConfiguration() {

	// Get ADC nb channels
	nbChannels = m_adcDriver->nbChannels();
	channelsActive.resize(nbChannels());
	triggerThreshold.resize(nbChannels());
	for(int32 i=0;i<nbChannels();++i) {
		channelsActive.update(i, m_adcDriver->channelActive.get(i));
		triggerThreshold.update(i, m_adcDriver->triggerThreshold.get(i));
	}
	registerRefresher(channelsActive, &CAENPksController::refreshChannelsActiveProperty, this);
	registerRefresher(triggerThreshold, &CAENPksController::refreshTriggerThresholdProperty, this);
}

/*
 * refreshChanneslActiveProperty
 */
void CAENPksController::refreshChannelsActiveProperty(int32 index, int32 aValue) throw (CannotSetValue) {
	m_adcDriver->channelActive.set(index, aValue);
	boost::thread td(boost::bind(&CAENPksController::writeParam, this));
}

/*
 * refreshTriggerThresholdProperty
 */
void CAENPksController::refreshTriggerThresholdProperty(int32 index, int32 aValue) throw (CannotSetValue) {
	m_adcDriver->triggerThreshold.set(index, aValue);
	boost::thread td(boost::bind(&CAENPksController::writeParam, this));
}

/*
 * writeParam
 */
void CAENPksController::writeParam() {
	m_adcDriver.execute(caen_pks::CAENPksDriver::WRITE_PARAMETERS_COMMAND, false);
}

}
