/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENCDSTATE_H
#define CAENCDSTATE_H

#include <Driver.h>

#include "CAENCdDriver.h"

namespace caen_cd {

/*!
 * \class CAENCdState
 * \brief Implements State pattern
 */
class CAENCdState: public DriverState<CAENCdDriver> {

public:

	/*!
	 * \brief Constructor.
	 */
	CAENCdState(CAENCdDriver* owner) :
		DriverState<CAENCdDriver> (owner) {}

	/*!
	 * \brief Destructor.
	 */
	virtual ~CAENCdState() {}

	/*!
	 * \brief Reset command implementation
	 */
	virtual void reset() = 0;

	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear() = 0;

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam() = 0;

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos() = 0;

	/*!
	 * \brief Set acquisition mode command implementation
	 */
	virtual void setAcqMode() = 0;
};

}

#endif //CAENCDSTATE_H
