/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALCAENCFDDRIVER_H
#define REALCAENCFDDRIVER_H

#include <CAENDigitizer.h>

#include "CAENCfdState.h"
#include "CAENCfdDef.h"

namespace caen_cfd {

class CAENCfdDriver;

/**
 * Real implementation
 */
class RealCAENCfdDriver : public CAENCfdState {

public:

	/**
	 * Constructor.
	 *
	 * @param Link to the class which used the state
	 */
	RealCAENCfdDriver(CAENCfdDriver* owner);

	/**
	 * Destructor.
	 * Empty.
	 */
	virtual ~RealCAENCfdDriver();

	/*!
	 * \brief Init command implementation
	 */
	virtual void init();

	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear();

	/*!
	 * \brief Reset command implementation
	 */
	virtual void reset();

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Set acquisition mode command implementation
	 */
	virtual void setAcqMode();

	/*!
	 * \brief Set channel enable command implementation
	 */
	virtual void setChannelEnable();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus();

	/*!
	 * \brief Enable Coincidence command implementation
	 */
	virtual void useCoincidence();

	/*!
	 * \brief Set CFD mode parameters
	 */
	virtual void setCfdParam();

	/*!
	 * \brief Set bit in register
	 */
	virtual void registerSetBits(uint16_t addr, int32 startBit, int32 endBit, int32 val);

private:

	void open();
	void close();

	int32 LockTempCalibration_x730(uint32_t ch);
	int32 ReadSPIRegister(uint32_t ch, uint32_t address, uint32_t *value);
	int32 WriteSPIRegister(uint32_t ch, uint32_t address, uint32_t value);

	int32 m_boardHandle;
	int32 m_boardInit;

//	CAEN_DGTZ_ErrorCode ret;
	int32 ret;

	CAEN_DGTZ_BoardInfo_t boardInfo;
	DigitizerParams_t boardParams;
	CAEN_DGTZ_DPP_PHA_Params_t dppParams;
};

}
#endif //REALCAENCFDDRIVER_H
