/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "../../../../../NomadSpecialModules/src/drivers/gigecam/photonics/SimulatedPhotonicsDriver.h"

#include <stdlib.h>
#include "tiffio.h"
#include "../../../../../NomadSpecialModules/src/drivers/gigecam/photonics/PhotonicsDef.h"
#include "../../../../../NomadSpecialModules/src/drivers/gigecam/photonics/PhotonicsDriver.h"

namespace photonics {

/*
 * Constructor
 */
SimulatedPhotonicsDriver::SimulatedPhotonicsDriver(PhotonicsDriver* owner) :
				PhotonicsState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
SimulatedPhotonicsDriver::~SimulatedPhotonicsDriver() {
	/* Empty */
}

void SimulatedPhotonicsDriver::init() {
}

/*
 * clearvme/daugther/Photonics
 */
void SimulatedPhotonicsDriver::clear() {
	// Read command
	owner()->time = 0;
	owner()->counts = 0;
	owner()->counts2 = 0;
	owner()->counts3 = 0;
}

/*
 * writeParam
 */
void SimulatedPhotonicsDriver::writeParam() {
}
//
///*
// * regroup
// */
//void SimulatedPhotonicsDriver::regroup() {
//}

/*
 * synchroniseRead
 */
void SimulatedPhotonicsDriver::synchroniseRead() {
	read();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void SimulatedPhotonicsDriver::read() {
	int32 size = owner()->dataSize();
	int32 *tab = owner()->data();
	float64 sum = 0;
	boost::filesystem::path file = owner()->fileSimu();
	TIFF* tif = TIFFOpen(file.string().c_str(), "r");
	if (tif) {
	  	int32 dircount = 0;
	  	while (dircount < owner()->indexSimu()) {
	  		if (TIFFReadDirectory(tif) <= 0) break;
	  		++dircount;
	  	};
	  	uint32 w, h;
	  	int32 npixels;
	  	tdata_t buf;

	  	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
	  	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
	  	npixels = w * h;
    	if (npixels == size) {
    		tmsize_t scanlinesize = TIFFScanlineSize(tif);
    		buf = _TIFFmalloc(scanlinesize);
			if (buf != NULL) {
				uint16* data;
				for (uint32 row = 0; row < h; row++) {
					TIFFReadScanline(tif, buf, row, 0);
					data = (uint16*) buf;
					for(uint32 i = 0; i < w;++i) {
						tab[row + i * h] = data[i];
						sum = sum + data[i];
					}
				}
				owner()->sum = sum;
				owner()->data.sendEvent();
				_TIFFfree(buf);
			}
    	}
	  	TIFFClose(tif);
	}

	// Read command
//	for (int32 i = 0; i < size; i++) {
//		tab[i] = i;
//		sum = sum + tab[i];
//	}
}

/*
 * pause 
 */
void SimulatedPhotonicsDriver::pause() {
}

/*
 * continue 
 */
void SimulatedPhotonicsDriver::resume() {
}

/*
 * start
 */
void SimulatedPhotonicsDriver::start() {
	owner()->time = owner()->time.setpoint();
}

/*
 * stop
 */
void SimulatedPhotonicsDriver::stop() {
}

/*
 * readStatus
 */
void SimulatedPhotonicsDriver::readStatus() {
	owner()->startActivated = false;
	// Status command
	owner()->computeGateStatus(photonics::READOUT_COMPLETE);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
	if (owner()->m_SynchroniseReadActivated == true) {
		if (owner()->m_SynchroniseReadTerminated == true) {
			owner()->m_SynchroniseReadActivated = false;
			owner()->m_SynchroniseReadTerminated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	} else {
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
}

/*
 * readInfos
 */
void SimulatedPhotonicsDriver::readInfos() {
}

}
