/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CADARACHEACQDRIVER_H
#define CADARACHEACQDRIVER_H

#include <Driver.h>
#include "drivers/global/Vme.h"
#include "controllers/common/acquisition/gate/KineticGateCommon.h"
#include "controllers/common/acquisition/mode/KineticAcquisitionCommon.h"
#include "controllers/common/acquisition/detector/DetectorCommon.h"

namespace cadaracheacq {

/*!
 * Global Implementation of the vme card Master acq
 */
class CadaracheAcqDriver: public driver::Vme,
		public acquisition::KineticGateCommon,
		public acquisition::KineticAcquisitionCommon,
		public acquisition::DetectorCommon {

	friend class RealCadaracheAcqDriver;
	friend class PerfectCadaracheAcqDriver;
	friend class SimulatedCadaracheAcqDriver;

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CadaracheAcqDriver(const std::string& aName);

	/*!
	 * \brief Destructor
	 */
	virtual ~CadaracheAcqDriver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	virtual void execute(const std::string& aCommand);

	/*!
	 * Properties
	 */

private:

	/*!
	 * Calculate the progression state.
	 */
	void calculateProgression();

	/*!
	 * \brief compute global gate status
	 * \param[in] value the driver status
	 */
	virtual void computeGateStatus(int32 value);

	/*!
	 * \brief compute global acquisition status
	 * \param[in] value the driver status
	 */
	virtual void computeAcquisitionStatus(int32 value);

	/**
	 * flag for synchronise read
	 */
	bool m_SynchroniseReadActivated;
	bool m_SynchroniseReadTerminated;

	bool m_StopActivated;

};

}
#endif //CADARACHEACQDRIVER_H
